from flume.procs.reduce.reducers.count import count
from flume.procs.reduce.reducers.iterate import iterate
from flume.procs.reduce.reducers.maximum import maximum
from flume.procs.reduce.reducers.minimum import minimum

from flume.procs.reduce.reducers.math import math
from flume.procs.reduce.reducers.date import date
from flume.procs.reduce.reducers.funcr import funcr
