from flume.sinks.memory import memory
from flume.sinks.write import write

from flume.sinks.barchart import barchart
from flume.sinks.linechart import linechart
from flume.sinks.timechart import timechart
